type
   PBuffer = ^TBuffer;
   TBuffer = array[0..63999] of byte;
   PLongBuffer = ^TLongBuffer;
   TLongBuffer = array[0..16000] of longint;
var
   image : array[1..2] of PBuffer;
   qwerty  : PLongBuffer;
   count : word;

const
   X_ADD1 : word = 0;
   Y_ADD1 : word = 150;
   X_ADD2 : word = 0;
   Y_ADD2 : word = 150;
   SCAN_CODE : char = #0;


procedure waitVerticalRetrace(count : word); assembler;
asm
      mov  cx, count
      jz   @@END
      mov  dx, 03DAh
@@JUMP1:
      in   al, dx
      test al, 08h
      jnz  @@JUMP1
@@JUMP2:
      in   al, dx
      test al, 08h
      jz   @@JUMP2
      loop @@JUMP1
@@END:
end;

function pressedKey : boolean; assembler;
asm
      mov  ah, 1
      int  $16
      jnz  @@KEY_PRESSED
      mov  al, FALSE
      jmp  @@END
@@KEY_PRESSED:
      mov  al, TRUE
@@END:
end;

function readKey : char; assembler;
asm
      mov  al, SCAN_CODE
      mov  SCAN_CODE, 0
      test al, $FF
      jnz  @@JUMP1
@@JUMP0:
      mov  ah, 1
      int  $16
      jz   @@JUMP0
      mov  ah, 0
      int  $16
      cmp  al, 0
      jne  @@JUMP1
      mov  SCAN_CODE, ah
@@JUMP1:
end;

procedure clearDevice;
begin
   waitVerticalRetrace(1);
   asm
              mov  es, SegA000
              xor  di, di
      db 66h; xor ax, ax
              mov  cx, 320 * 200 / 4
              cld
      db 66h; rep  stosw
   end;
end;

procedure makePalette(r1, g1, b1, r2, g2, b2, _begin, _end : byte);
const
   divISION = 256;
var
   i : integer;
   red, green, blue, r_inc, g_inc, b_inc : longint;
begin
   r_inc := longint(succ(r2-r1)) * divISION div succ(_end-_begin);
   g_inc := longint(succ(g2-g1)) * divISION div succ(_end-_begin);
   b_inc := longint(succ(b2-b1)) * divISION div succ(_end-_begin);
   red   := longint(r1) * divISION;
   green := longint(g1) * divISION;
   blue  := longint(b1) * divISION;
   asm
         mov  al, _begin
         mov  dx, 03C8h
         out  dx, al
   end;
   for i := _begin to _end do begin
      asm
                 mov  dx, 03C9h
         db 66h; mov  ax, word ptr red
         db 66h; shr  ax, 8
                 out  dx, al
         db 66h; mov  ax, word ptr green
         db 66h; shr  ax, 8
                 out  dx, al
         db 66h; mov  ax, word ptr blue
         db 66h; shr  ax, 8
                 out  dx, al
      end;
      inc(red,r_inc);
      inc(green,g_inc);
      inc(blue,b_inc);
   end;
end;

procedure putImage(x,y : word; var image; target : pointer; skip_bit : byte);
var
   x_size : word;
   pbuffer, plid : pointer;
begin
   pbuffer := target;
   asm
      push ds
      lds si, image
      les di, pbuffer
      cld
      lodsw
      mov x_size, ax
      lodsw
      mov cx, ax

      mov ax, y
      shl ax, 2
      add ax, y
      shl ax, 6
      add ax, x
      add di, ax
   @MAIN_LOOP:
      push cx
      mov cx, x_size
   @SUB_LOOP:
      mov al, ds:[si]
      cmp al, skip_bit
      jz  @SKIP
      mov es:[di], al
   @SKIP:
      inc si
      inc di
      loop @SUB_LOOP
      add di, 320
      sub di, x_size
      pop cx
      loop @MAIN_LOOP
      pop ds
   end;
end;

procedure updateMorphing(source, dest : pointer; step : integer);
var
   x, y, _x, _y : integer;
   x_size1, x_size2, y_size1, y_size2 : word;
   image1, image2 : PBuffer;
   _qwerty : PLongBuffer;

 procedure moveNext(pb : PBuffer; var x,y : integer);
 begin
    repeat
       inc(x);
       if (x >= x_size2+X_ADD2) then begin
          inc(y);
          x := X_ADD2;
          if (y >= y_size2+Y_ADD2) then y := Y_ADD2;
       end;
    until (pb^[(y-Y_ADD2)*x_size2+x-X_ADD2] <> $FF);
 end;

begin
   _qwerty := qwerty;

   image1 := source;
   move(image1^,x_size1,2); inc(word(image1),2);
   move(image1^,y_size1,2); inc(word(image1),2);
   image2 := dest;
   move(image2^,x_size2,2); inc(word(image2),2);
   move(image2^,y_size2,2); inc(word(image2),2);
   count:=0;
   _x := X_ADD2; _y := Y_ADD2;
   moveNext(image2,_x,_y);

   for x := X_ADD1 to pred(x_size1)+X_ADD1 do begin
    for y := Y_ADD1 to pred(y_size1)+Y_ADD1 do begin
      if (image1^[(y-Y_ADD1)*x_size1+x-X_ADD1] <> $FF) then begin
         _qwerty^[0] := (longint(_y-y) shl 16) div step;
         _qwerty^[1] := longint(y) shl 16;
         _qwerty^[2] := (longint(_x-x) shl 16) div step;
         _qwerty^[3] := longint(x) shl 16;
         _qwerty^[4] := (longint(image2^[(_y-Y_ADD2)*x_size2+_x-X_ADD2]-image1^[(y-Y_ADD1)*x_size1+x-X_ADD1]) shl 16) div step;
         _qwerty^[5] := longint(image1^[(y-Y_ADD1)*x_size1+x-X_ADD1]) shl 16;
         moveNext(image2,_x,_y);
         inc(word(_qwerty),6*4);
         inc(count);
      end;
    end;
   end;
end;

procedure Morphing; assembler;
asm
                push ds
                mov  es, SegA000
                mov  cx, count
                lds  si, qwerty
                cld
@@LOOP:
        db 66h; lodsw
        db 66h; add  [si],ax
        db 66h; lodsw
        db 66h; shr  ax, 16
                mov  bx, 320
                mul  bx
                mov  di, ax

        db 66h; lodsw
        db 66h; add  [si],ax
        db 66h; lodsw
        db 66h; shr  ax, 16
                add  di,ax

        db 66h; lodsw
        db 66h; add  [si],ax
        db 66h; lodsw
        db 66h; shr  ax, 16
                mov  es:[di],al

                loop @@LOOP
                pop  ds
end;

procedure initProgram;
var
   i : integer;
   f : file;
   pb : PBuffer;
begin
   assign(f,'z1.img');
   reset(f,1);
   getMem(image[1],FileSize(f));
   BlockRead(f,image[1]^,FileSize(f));
   close(f);

   assign(f,'z2.img');
   reset(f,1);
   getMem(image[2],FileSize(f));
   BlockRead(f,image[2]^,FileSize(f));
   close(f);

   asm
      mov  ax, 13h
      int  10h
   end;

   assign(f,'z.pal');
   reset(f,1);
   getMem(pb,256*3);
   BlockRead(f,pb^,256*3);
   close(f);

   port[$3C8] := 0;
   for i := 0 to pred(256*3) do port[$3C9] := pb^[i];

   freeMem(pb,256*3);

   new(qwerty);

   makePalette(0,0,0,44,24,53,0,127);

end;

procedure excuteProgram;
var
   i : word;
   pb : PBuffer;
begin
   while not pressedKey do begin
      updateMorphing(image[1],image[2],100);
      putImage(X_ADD1,Y_ADD1,image[1]^,ptr(SegA000,0),$FF);
      waitVerticalRetrace(60);
      if pressedKey then break;
      for i := 1 to 100 do begin
         if pressedKey then break;
         clearDevice;
         Morphing;
      end;
      if pressedKey then break;
      clearDevice;
      putImage(X_ADD2,Y_ADD2,image[2]^,ptr(SegA000,0),$FF);
      waitVerticalRetrace(60);
      X_ADD1 := X_ADD2;
      Y_ADD1 := Y_ADD2;
      X_ADD2 := random(200);
      Y_ADD2 := random(80)+100;
      pb := image[1]; image[1] := image[2]; image[2] := pb;
   end;
   readKey;
end;

procedure closeProgram;
begin
   asm
      mov  ax, 03h
      int  10h
   end;
end;

begin
   initProgram;
   excuteProgram;
   closeProgram;
end.